@@ -16,6 +16,7 @@
       2020-08-12 added headerR option
       2020-08-26 readSpectrum backend
       2021-05-03 reorder xic arguments
+      2025-05-27 https://github.com/fgcz/rawrr/issues/84
     */
 
     using System;
@@ -164,27 +165,30 @@
 	    }
 	    }
 
+
             public static void GetIndex(this IRawDataPlus rawFile){
 	            int firstScanNumber = rawFile.RunHeaderEx.FirstSpectrum;
 	            int lastScanNumber = rawFile.RunHeaderEx.LastSpectrum;
 
-	            int idxCharge = rawFile.GetIndexOfPattern("Charge State");
-	            int idxMasterScan = rawFile.GetIndexOfPattern("Master Scan Number:");
-	            int idxDependencyType = rawFile.GetIndexOfPattern("Dependency Type:");
-		    int idxMonoisotopicmZ = rawFile.GetIndexOfPattern("Monoisotopic M/Z:");
-
 	            double charge, precursorMass;
 		    double monoIsotopicMz;
 	            int masterScan, dependencyType;
 
 	            Console.WriteLine("scan;scanType;StartTime;precursorMass;MSOrder;charge;masterScan;dependencyType;monoisotopicMz");
 
+                    Dictionary<string, string> ScanTrailerDict;
+
 		    foreach (int scanNumber in Enumerable.Range(firstScanNumber, lastScanNumber)){
 		            var scanTrailer = rawFile.GetTrailerExtraInformation(scanNumber);
 		            var scanStatistics = rawFile.GetScanStatsForScanNumber(scanNumber);
 		            var scanEvent = rawFile.GetScanEventForScanNumber(scanNumber);
 		            var scanFilter = rawFile.GetFilterForScanNumber(scanNumber);
 
+                	// TODO(cpanse): implement a public class ScanTrailer
+                	ScanTrailerDict = new Dictionary<string, string>();
+			foreach (var (key, value) in Enumerable.Range(0, scanTrailer.Length).Select(i => (scanTrailer.Labels[i], scanTrailer.Values[i])))
+			{ ScanTrailerDict[key] = value.Trim(); }
+
 		            try{
 			            var reaction0 = scanEvent.GetReaction(0);
 			            precursorMass =  reaction0.PrecursorMass;
@@ -193,29 +197,27 @@
 		            }
 
 		            try{
-			            charge = int.Parse(scanTrailer.Values.ToArray()[idxCharge]);
+			            charge = int.Parse(ScanTrailerDict["Charge State:"]);
 		            } catch {
 			            charge = -1;
 		            }
 
 		            try{
-			            masterScan = int.Parse(scanTrailer.Values.ToArray()[idxMasterScan]);
+			            masterScan = int.Parse(ScanTrailerDict["Master Scan Number:"]);
 		            } catch {
 			            masterScan= -1;
 		            }
 
 		            try{
-			            dependencyType = int.Parse(scanTrailer.Values.ToArray()[idxDependencyType]);
+			            dependencyType = int.Parse(ScanTrailerDict["Dependency Type:"]);
 		            } catch {
 			            dependencyType = -1;
 		            }
 
 		            try{
-                                monoIsotopicMz = Convert.ToDouble(scanTrailer.Values.ToArray()[idxMonoisotopicmZ]);
-				//monoisotopicMz = 0.0;
+                                monoIsotopicMz = Convert.ToDouble(ScanTrailerDict["Monoisotopic M/Z:"]);
 			    } catch {
 				monoIsotopicMz = -1.0;
-			        //monoisotopicMz = -1;
 			    }
 
 		            Console.WriteLine("{0};{1};{2};{3};{4};{5};{6};{7};{8}", scanNumber,
@@ -230,35 +232,13 @@
 	            }
             }
 
-	    private static int GetIndexOfPattern(this IRawDataPlus rawFile, string pattern="Charge State"){
-                    var trailerFields = rawFile.GetTrailerExtraHeaderInformation();
-
-		    int idx = -1;
-                        try
-                        {
-                            idx = trailerFields
-                                .Select((item, index) => new
-                                {
-                                    name = item.Label.ToString(),
-                                    Position = index
-                                })
-                                .First(x => x.name.Contains(pattern)).Position;
-                        }
-                        catch
-                        {
-		        }
-			return (idx);
-	    }
-
             public static void WriteSpectrumAsRcode0(this IRawDataPlus rawFile, string filename)
             {
-
-
-		    int idxCharge = rawFile.GetIndexOfPattern();
              	    int firstScanNumber = rawFile.RunHeaderEx.FirstSpectrum;
             	    int lastScanNumber = rawFile.RunHeaderEx.LastSpectrum;
-		        int charge = -1;
-		        double pc=-1;
+		    int charge = -1;
+		    double pc=-1;
+                    Dictionary<string, string> ScanTrailerDict;
 
                 using (System.IO.StreamWriter file =
                     new System.IO.StreamWriter(filename))
@@ -269,20 +249,24 @@
                         var scanEvent = rawFile.GetScanEventForScanNumber(scanNumber);
 			var scanFilter = rawFile.GetFilterForScanNumber(scanNumber);
 
+		        ScanTrailerDict = new Dictionary<string, string>();
+			foreach (var (key, value) in Enumerable.Range(0, scanTrailer.Length).Select(i => (scanTrailer.Labels[i], scanTrailer.Values[i])))
+			{ ScanTrailerDict[key] = value.Trim(); }
+
 
 		        try{
-                        var reaction0 = scanEvent.GetReaction(0);
-		        pc =  reaction0.PrecursorMass;
+                          var reaction0 = scanEvent.GetReaction(0);
+		          pc =  reaction0.PrecursorMass;
 		        }
 		        catch{
-			        pc = -1;
+			  pc = -1;
 		        }
 
 		        try{
-                    	    charge = int.Parse(scanTrailer.Values.ToArray()[idxCharge]);
+                    	    charge = int.Parse(ScanTrailerDict["Charge State:"]);
                         }
 		        catch {
-			        charge=-1;
+			        charge = -1;
 		        }
 
                         file.WriteLine("e$Spectrum[[{0}]] <- list(", scanNumber);
@@ -310,8 +294,9 @@
             public static void WriteCentroidSpectrumAsRcode(this IRawDataPlus rawFile, string filename, List<int> L)
             {
                 int count = 1;
+		int charge = -1;
+                Dictionary<string, string> ScanTrailerDict;
                 var trailerFields = rawFile.GetTrailerExtraHeaderInformation();
-		            int indexCharge = rawFile.GetIndexOfPattern("Charge State");
 
                 using (System.IO.StreamWriter file =
                     new System.IO.StreamWriter(filename))
@@ -324,12 +309,24 @@
                         var scanEvent = rawFile.GetScanEventForScanNumber(scanNumber);
                         var scanTrailer = rawFile.GetTrailerExtraInformation(scanNumber);
 
+		        ScanTrailerDict = new Dictionary<string, string>();
+			foreach (var (key, value) in Enumerable.Range(0, scanTrailer.Length).Select(i => (scanTrailer.Labels[i], scanTrailer.Values[i])))
+			{ ScanTrailerDict[key] = value.Trim(); }
+
+		        try{
+                    	    charge = int.Parse(ScanTrailerDict["Charge State:"]);
+                        }
+		        catch {
+			        charge = -1;
+		        }
+
+
                         file.WriteLine("e$Spectrum[[{0}]] <- list(", count++);
                         file.WriteLine("\tscan = {0},", scanNumber);
                         file.WriteLine("\tStartTime = {0},", scanStatistics.StartTime);
                         file.WriteLine("\trtinseconds = {0},", Math.Round(scanStatistics.StartTime * 60 * 1000) / 1000);
-                        if (indexCharge > 0)
-                                file.WriteLine("\tcharge = {0},", int.Parse(scanTrailer.Values.ToArray()[indexCharge]));
+                        if (charge > 0)
+                                file.WriteLine("\tcharge = {0},", charge);
 			    else
                                 file.WriteLine("\tcharge = NA,");
 
@@ -342,6 +339,8 @@
 
                         if (scanStatistics.IsCentroidScan && centroidStream.Length > 0)
                         {
+                                file.WriteLine("\tcentroidScan.mZ = c(" + string.Join(", ", scan.CentroidScan.Masses.ToArray()) + "),");
+                                file.WriteLine("\tcentroidScan.intensity = c(" + string.Join(", ", scan.CentroidScan.Intensities.ToArray()) + "),");
                         	file.WriteLine("\tmZ = c(" + string.Join(", ", centroidStream.Masses) + "),");
                                 file.WriteLine("\tintensity = c(" + string.Join(", ", centroidStream.Intensities) + ")");
 			} else{
@@ -365,18 +364,21 @@
 
             public static void WriteTrailerValues(this IRawDataPlus rawFile, string label)
 	    {
-		    int idx = -1;
-		    try{
-		    	idx = rawFile.GetIndexOfPattern(label);
-		    }catch (Exception ex){
-                        Console.WriteLine("GetIndexOfPattern {} caused an exception {}.", label, ex.Message);
-                        return;
-		    }
-
+                Dictionary<string, string> ScanTrailerDict;
 		    foreach (int scanNumber in Enumerable.Range(rawFile.RunHeaderEx.FirstSpectrum, rawFile.RunHeaderEx.LastSpectrum))
                     {
                         var scanTrailer = rawFile.GetTrailerExtraInformation(scanNumber);
-                    	Console.WriteLine(scanTrailer.Values.ToArray()[idx]);
+
+                	// TODO(cpanse): implement a public class ScanTrailer 
+                	ScanTrailerDict = new Dictionary<string, string>();
+			foreach (var (key, value) in Enumerable.Range(0, scanTrailer.Length).Select(i => (scanTrailer.Labels[i], scanTrailer.Values[i])))
+			{ ScanTrailerDict[key] = value.Trim(); }
+
+			if (ScanTrailerDict.ContainsKey(label)){
+                    		Console.WriteLine(ScanTrailerDict[label]);
+			}else{
+                    		Console.WriteLine("NA");
+			}
 		    }
 	    }
 
@@ -388,9 +390,10 @@
             public static void WriteSpectrumAsRcode(this IRawDataPlus rawFile, string filename, List<int> L)
             {
                 int count = 1;
+                int charge = -1;
+                double monoIsotopicMz = -1;
                 var trailerFields = rawFile.GetTrailerExtraHeaderInformation();
-                int indexCharge = rawFile.GetIndexOfPattern("Charge State");
-		int indexMonoisotopicmZ = rawFile.GetIndexOfPattern("MonoisotopicmZ");
+                Dictionary<string, string> ScanTrailerDict;
 
                 using (System.IO.StreamWriter file =
                     new System.IO.StreamWriter(filename))
@@ -407,6 +410,23 @@
                         var scanEvent = rawFile.GetScanEventForScanNumber(scanNumber);
 
                         var scan = Scan.FromFile(rawFile, scanNumber);
+ 
+
+		        ScanTrailerDict = new Dictionary<string, string>();
+			foreach (var (key, value) in Enumerable.Range(0, scanTrailer.Length).Select(i => (scanTrailer.Labels[i], scanTrailer.Values[i])))
+			{ ScanTrailerDict[key] = value.Trim(); }
+
+		        try{
+			        charge = int.Parse(ScanTrailerDict["Charge State:"]);
+		        } catch {
+			        charge = -1;
+		        }
+
+		        try{
+                            monoIsotopicMz = Convert.ToDouble(ScanTrailerDict["Monoisotopic M/Z:"]);
+			} catch {
+			    monoIsotopicMz = -1.0;
+			}
 
                         file.WriteLine("e$Spectrum[[{0}]] <- list(", count++);
                         file.WriteLine("\tscan = {0},", scanNumber);
@@ -452,16 +472,19 @@
                                     null,
                                     scanNumber);
 
-			    if (indexMonoisotopicmZ > 0)
-                                file.WriteLine("\tmonoisotopicMz = {0},", Convert.ToDouble(scanTrailer.Values.ToArray()[indexMonoisotopicmZ]));
+			    if (monoIsotopicMz > 0)
+                                file.WriteLine("\tmonoisotopicMz = {0},", monoIsotopicMz);
 			    else
                                 file.WriteLine("\tmonoisotopicMz = NA,");
 
 
-			    if (indexCharge > 0)
-                                file.WriteLine("\tcharge = {0},", int.Parse(scanTrailer.Values.ToArray()[indexCharge]));
-			    else
+			    if (charge > 0){
+                                file.WriteLine("\tcharge = {0},", charge);
+			    }
+                            else{
                                 file.WriteLine("\tcharge = NA,");
+                            }
+	
 
                                 file.WriteLine("\tmZ = c(" + string.Join(", ", centroidStream.Masses) + "),");
                                 file.WriteLine("\tintensity = c(" + string.Join(", ", centroidStream.Intensities) + "),");
@@ -491,13 +514,14 @@
                                     null,
                                     scanNumber);
 
-			    if (indexCharge > 0)
-                                file.WriteLine("\tcharge = {0},", int.Parse(scanTrailer.Values.ToArray()[indexCharge]));
+
+			    if (charge > 0)
+                                file.WriteLine("\tcharge = {0},", charge);
 			    else
                                 file.WriteLine("\tcharge = NA,");
 
-			    if (indexMonoisotopicmZ > 0)
-                                file.WriteLine("\tmonoisotopicMz = {0},", Convert.ToDouble(scanTrailer.Values.ToArray()[indexMonoisotopicmZ]));
+			    if (monoIsotopicMz > 0)
+                                file.WriteLine("\tmonoisotopicMz = {0},", monoIsotopicMz);
 			    else
                                 file.WriteLine("\tmonoisotopicMz = NA,");
 
@@ -531,7 +555,7 @@
 	        {
 		        // This local variable controls if the AnalyzeAllScans method is called
 		        // bool analyzeScans = false;
-		        const string rawR_version = "1.3.1";
+		        const string rawrr_version = "1.17.2";
 		        string filename = string.Empty;
 		        string mode = string.Empty;
 		        string filterString = string.Empty;
@@ -547,11 +571,12 @@
 				        "Extracts filtered (option 2) ion chromatograms within a given mass and mass tolerance [in ppm] (option 3) xic of a given raw file as R code into a file."
 			        },
 			        {"scans", "Extracts scans (spectra) of a given ID as Rcode."},
-			        {"cscans", "Extracts 'barbone' scans (spectra), including only mZ, intensity , precursorMass, rtinsecodonds and charge state, of a given ID as Rcode."},
+			        {"barebone", "Extracts 'barebone' scans (spectra), including only mZ, intensity , precursorMass, rtinsecodonds and charge state, of a given ID as Rcode."},
 			        {"index", "Prints index as csv of all scans."},
 			        {"trailer", "Prints all trailer labels."}
 		        };
 		        var helpOptions = new List<string>() {"help", "--help", "-h", "h", "/h"};
+		        var versionOptions = new List<string>() {"version", "--version", "-v", "-V", "/v"};
 
 		        if (args.Length >= 2){
 			        filename = args[0];
@@ -571,14 +596,12 @@
 			        {
 				    Console.WriteLine("No RAW file specified!");
 			            return;
-				        //Console.WriteLine("run 'rawrr.exe help'.");
-				        //Environment.Exit(1);
 			        }
-			        else if (args[0] == "version")
-			        {
-				        Console.WriteLine(rawR_version);
+			        else if (versionOptions.Contains(args[0]))
+                                {
+				        Console.WriteLine(rawrr_version);
 				        Environment.Exit(0);
-			        }
+                                }
 			        else if (helpOptions.Contains(args[0]))
 			        {
 				        Console.WriteLine("\nUsage:\n");
@@ -705,12 +728,6 @@
                         Environment.Exit(0);
                     }
 
-                    if (mode == "version")
-                    {
-                         Console.WriteLine("version={}", rawR_version);
-                         Environment.Exit(0);
-                    }
-
                     if (mode == "chromatogram")
                     {
                         // Get the BasePeak chromatogram for the MS data
@@ -762,7 +779,8 @@
                         return;
 
                     }
-                    if (mode == "cscans")
+                    // extracs only the specta 
+                    if (mode == "barebone")
                     {
                         List<int> scans = new List<int>();
 
@@ -772,6 +790,7 @@
                         foreach (var line in File.ReadAllLines(scanfile))
                         {
 
+                            // parses the input while accepting only integers greater than 0
 			    try{
                             Int32.TryParse(line, out scanNumber);
 			    if (scanNumber > 0)
